"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    engine: str,
    server_name: str,
    instance_profile_arn: str,
    instance_type: str,
    service_role_arn: str,
    associate_public_ip_address: bool = None,
    custom_domain: str = None,
    custom_certificate: str = None,
    custom_private_key: str = None,
    disable_automated_backup: bool = None,
    engine_model: str = None,
    engine_version: str = None,
    engine_attributes: List = None,
    backup_retention_count: int = None,
    key_pair: str = None,
    preferred_maintenance_window: str = None,
    preferred_backup_window: str = None,
    security_group_ids: List = None,
    subnet_ids: List = None,
    tags: List = None,
    backup_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Creates and immedately starts a new server. The server is ready to use when it is in the HEALTHY state. By default, you can create a maximum of 10 servers.   This operation is asynchronous.   A LimitExceededException is thrown when you have created the maximum number of servers (10). A ResourceAlreadyExistsException is thrown when a server with the same name already exists in the account. A ResourceNotFoundException is thrown when you specify a backup ID that is not valid or is for a backup that does not exist. A ValidationException is thrown when parameters of the request are not valid.   If you do not specify a security group by adding the SecurityGroupIds parameter, AWS OpsWorks creates a new security group.   Chef Automate: The default security group opens the Chef server to the world on TCP port 443. If a KeyName is present, AWS OpsWorks enables SSH access. SSH is also open to the world on TCP port 22.   Puppet Enterprise: The default security group opens TCP ports 22, 443, 4433, 8140, 8142, 8143, and 8170. If a KeyName is present, AWS OpsWorks enables SSH access. SSH is also open to the world on TCP port 22.  By default, your server is accessible from any IP address. We recommend that you update your security group rules to allow access from known IP addresses and address ranges only. To edit security group rules, open Security Groups in the navigation pane of the EC2 management console.  To specify your own domain for a server, and provide your own self-signed or CA-signed certificate and private key, specify values for CustomDomain, CustomCertificate, and CustomPrivateKey.

    Args:
        associate_public_ip_address(bool, optional):  Associate a public IP address with a server that you are launching. Valid values are true or false. The default value is true. . Defaults to None.
        custom_domain(str, optional): An optional public endpoint of a server, such as https://aws.my-company.com. To access the server, create a CNAME DNS record in your preferred DNS service that points the custom domain to the endpoint that is generated when the server is created (the value of the CreateServer Endpoint attribute). You cannot access the server by using the generated Endpoint value if the server is using a custom domain. If you specify a custom domain, you must also specify values for CustomCertificate and CustomPrivateKey. Defaults to None.
        custom_certificate(str, optional): A PEM-formatted HTTPS certificate. The value can be be a single, self-signed certificate, or a certificate chain. If you specify a custom certificate, you must also specify values for CustomDomain and CustomPrivateKey. The following are requirements for the CustomCertificate value:   You can provide either a self-signed, custom certificate, or the full certificate chain.   The certificate must be a valid X509 certificate, or a certificate chain in PEM format.   The certificate must be valid at the time of upload. A certificate can't be used before its validity period begins (the certificate's NotBefore date), or after it expires (the certificate's NotAfter date).   The certificate’s common name or subject alternative names (SANs), if present, must match the value of CustomDomain.   The certificate must match the value of CustomPrivateKey.  . Defaults to None.
        custom_private_key(str, optional): A private key in PEM format for connecting to the server by using HTTPS. The private key must not be encrypted; it cannot be protected by a password or passphrase. If you specify a custom private key, you must also specify values for CustomDomain and CustomCertificate. Defaults to None.
        disable_automated_backup(bool, optional):  Enable or disable scheduled backups. Valid values are true or false. The default value is true. . Defaults to None.
        engine(str):  The configuration management engine to use. Valid values include ChefAutomate and Puppet. .
        engine_model(str, optional):  The engine model of the server. Valid values in this release include Monolithic for Puppet and Single for Chef. . Defaults to None.
        engine_version(str, optional):  The major release version of the engine that you want to use. For a Chef server, the valid value for EngineVersion is currently 2. For a Puppet server, valid values are 2019 or 2017. . Defaults to None.
        engine_attributes(List, optional): Optional engine attributes on a specified server.   Attributes accepted in a Chef createServer request:     CHEF_AUTOMATE_PIVOTAL_KEY: A base64-encoded RSA public key. The corresponding private key is required to access the Chef API. When no CHEF_AUTOMATE_PIVOTAL_KEY is set, a private key is generated and returned in the response.     CHEF_AUTOMATE_ADMIN_PASSWORD: The password for the administrative user in the Chef Automate web-based dashboard. The password length is a minimum of eight characters, and a maximum of 32. The password can contain letters, numbers, and special characters (!/@#$%^&+=_). The password must contain at least one lower case letter, one upper case letter, one number, and one special character. When no CHEF_AUTOMATE_ADMIN_PASSWORD is set, one is generated and returned in the response.    Attributes accepted in a Puppet createServer request:     PUPPET_ADMIN_PASSWORD: To work with the Puppet Enterprise console, a password must use ASCII characters.    PUPPET_R10K_REMOTE: The r10k remote is the URL of your control repository (for example, ssh://git@your.git-repo.com:user/control-repo.git). Specifying an r10k remote opens TCP port 8170.    PUPPET_R10K_PRIVATE_KEY: If you are using a private Git repository, add PUPPET_R10K_PRIVATE_KEY to specify a PEM-encoded private SSH key.  . Defaults to None.
        backup_retention_count(int, optional):  The number of automated backups that you want to keep. Whenever a new backup is created, AWS OpsWorks CM deletes the oldest backups if this number is exceeded. The default value is 1. . Defaults to None.
        server_name(str):  The name of the server. The server name must be unique within your AWS account, within each region. Server names must start with a letter; then letters, numbers, or hyphens (-) are allowed, up to a maximum of 40 characters. .
        instance_profile_arn(str):  The ARN of the instance profile that your Amazon EC2 instances use. Although the AWS OpsWorks console typically creates the instance profile for you, if you are using API commands instead, run the service-role-creation.yaml AWS CloudFormation template, located at https://s3.amazonaws.com/opsworks-cm-us-east-1-prod-default-assets/misc/opsworks-cm-roles.yaml. This template creates a CloudFormation stack that includes the instance profile you need. .
        instance_type(str):  The Amazon EC2 instance type to use. For example, m5.large. .
        key_pair(str, optional):  The Amazon EC2 key pair to set for the instance. This parameter is optional; if desired, you may specify this parameter to connect to your instances by using SSH. . Defaults to None.
        preferred_maintenance_window(str, optional):  The start time for a one-hour period each week during which AWS OpsWorks CM performs maintenance on the instance. Valid values must be specified in the following format: DDD:HH:MM. MM must be specified as 00. The specified time is in coordinated universal time (UTC). The default value is a random one-hour period on Tuesday, Wednesday, or Friday. See TimeWindowDefinition for more information.   Example: Mon:08:00, which represents a start time of every Monday at 08:00 UTC. (8:00 a.m.) . Defaults to None.
        preferred_backup_window(str, optional):  The start time for a one-hour period during which AWS OpsWorks CM backs up application-level data on your server if automated backups are enabled. Valid values must be specified in one of the following formats:     HH:MM for daily backups    DDD:HH:MM for weekly backups    MM must be specified as 00. The specified time is in coordinated universal time (UTC). The default value is a random, daily start time.  Example: 08:00, which represents a daily start time of 08:00 UTC.  Example: Mon:08:00, which represents a start time of every Monday at 08:00 UTC. (8:00 a.m.). Defaults to None.
        security_group_ids(List, optional):  A list of security group IDs to attach to the Amazon EC2 instance. If you add this parameter, the specified security groups must be within the VPC that is specified by SubnetIds.   If you do not specify this parameter, AWS OpsWorks CM creates one new security group that uses TCP ports 22 and 443, open to 0.0.0.0/0 (everyone). . Defaults to None.
        service_role_arn(str):  The service role that the AWS OpsWorks CM service backend uses to work with your account. Although the AWS OpsWorks management console typically creates the service role for you, if you are using the AWS CLI or API commands, run the service-role-creation.yaml AWS CloudFormation template, located at https://s3.amazonaws.com/opsworks-cm-us-east-1-prod-default-assets/misc/opsworks-cm-roles.yaml. This template creates a CloudFormation stack that includes the service role and instance profile that you need. .
        subnet_ids(List, optional):  The IDs of subnets in which to launch the server EC2 instance.   Amazon EC2-Classic customers: This field is required. All servers must run within a VPC. The VPC must have "Auto Assign Public IP" enabled.   EC2-VPC customers: This field is optional. If you do not specify subnet IDs, your EC2 instances are created in a default subnet that is selected by Amazon EC2. If you specify subnet IDs, the VPC must have "Auto Assign Public IP" enabled.  For more information about supported Amazon EC2 platforms, see Supported Platforms. Defaults to None.
        tags(List, optional): A map that contains tag keys and tag values to attach to an AWS OpsWorks for Chef Automate or AWS OpsWorks for Puppet Enterprise server.   The key cannot be empty.   The key can be a maximum of 127 characters, and can contain only Unicode letters, numbers, or separators, or the following special characters: + - = . _ : / @    The value can be a maximum 255 characters, and contain only Unicode letters, numbers, or separators, or the following special characters: + - = . _ : / @    Leading and trailing white spaces are trimmed from both the key and value.   A maximum of 50 user-applied tags is allowed for any AWS OpsWorks-CM server.  . Defaults to None.
        backup_id(str, optional):  If you specify this field, AWS OpsWorks CM creates the server by using the backup represented by BackupId. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.opsworkscm.server.init.create(
                ctx,
                engine=value,
                server_name=value,
                instance_profile_arn=value,
                instance_type=value,
                service_role_arn=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.opsworkscm.server.init.create engine=value, server_name=value, instance_profile_arn=value, instance_type=value, service_role_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="opsworkscm",
        operation="create_server",
        op_kwargs={
            "AssociatePublicIpAddress": associate_public_ip_address,
            "CustomDomain": custom_domain,
            "CustomCertificate": custom_certificate,
            "CustomPrivateKey": custom_private_key,
            "DisableAutomatedBackup": disable_automated_backup,
            "Engine": engine,
            "EngineModel": engine_model,
            "EngineVersion": engine_version,
            "EngineAttributes": engine_attributes,
            "BackupRetentionCount": backup_retention_count,
            "ServerName": server_name,
            "InstanceProfileArn": instance_profile_arn,
            "InstanceType": instance_type,
            "KeyPair": key_pair,
            "PreferredMaintenanceWindow": preferred_maintenance_window,
            "PreferredBackupWindow": preferred_backup_window,
            "SecurityGroupIds": security_group_ids,
            "ServiceRoleArn": service_role_arn,
            "SubnetIds": subnet_ids,
            "Tags": tags,
            "BackupId": backup_id,
        },
    )


async def delete(hub, ctx, server_name: str) -> Dict:
    r"""
    **Autogenerated function**

     Deletes the server and the underlying AWS CloudFormation stacks (including the server's EC2 instance). When you run this command, the server state is updated to DELETING. After the server is deleted, it is no longer returned by DescribeServer requests. If the AWS CloudFormation stack cannot be deleted, the server cannot be deleted.   This operation is asynchronous.   An InvalidStateException is thrown when a server deletion is already in progress. A ResourceNotFoundException is thrown when the server does not exist. A ValidationException is raised when parameters of the request are not valid.

    Args:
        server_name(str): The ID of the server to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.opsworkscm.server.init.delete(ctx, server_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.opsworkscm.server.init.delete server_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="opsworkscm",
        operation="delete_server",
        op_kwargs={"ServerName": server_name},
    )


async def describe_all(
    hub, ctx, server_name: str = None, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

     Lists all configuration management servers that are identified with your account. Only the stored results from Amazon DynamoDB are returned. AWS OpsWorks CM does not query other services.   This operation is synchronous.   A ResourceNotFoundException is thrown when the server does not exist. A ValidationException is raised when parameters of the request are not valid.

    Args:
        server_name(str, optional): Describes the server with the specified ServerName. Defaults to None.
        next_token(str, optional): This is not currently implemented for DescribeServers requests. . Defaults to None.
        max_results(int, optional): This is not currently implemented for DescribeServers requests. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.opsworkscm.server.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.opsworkscm.server.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="opsworkscm",
        operation="describe_servers",
        op_kwargs={
            "ServerName": server_name,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )


async def restore(
    hub,
    ctx,
    backup_id: str,
    server_name: str,
    instance_type: str = None,
    key_pair: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Restores a backup to a server that is in a CONNECTION_LOST, HEALTHY, RUNNING, UNHEALTHY, or TERMINATED state. When you run RestoreServer, the server's EC2 instance is deleted, and a new EC2 instance is configured. RestoreServer maintains the existing server endpoint, so configuration management of the server's client devices (nodes) should continue to work.  Restoring from a backup is performed by creating a new EC2 instance. If restoration is successful, and the server is in a HEALTHY state, AWS OpsWorks CM switches traffic over to the new instance. After restoration is finished, the old EC2 instance is maintained in a Running or Stopped state, but is eventually terminated.  This operation is asynchronous.   An InvalidStateException is thrown when the server is not in a valid state. A ResourceNotFoundException is thrown when the server does not exist. A ValidationException is raised when parameters of the request are not valid.

    Args:
        backup_id(str):  The ID of the backup that you want to use to restore a server. .
        server_name(str):  The name of the server that you want to restore. .
        instance_type(str, optional):  The type of instance to restore. Valid values must be specified in the following format: ^([cm][34]|t2).* For example, m5.large. Valid values are m5.large, r5.xlarge, and r5.2xlarge. If you do not specify this parameter, RestoreServer uses the instance type from the specified backup. . Defaults to None.
        key_pair(str, optional):  The name of the key pair to set on the new EC2 instance. This can be helpful if the administrator no longer has the SSH key. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.opsworkscm.server.init.restore(
                ctx, backup_id=value, server_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.opsworkscm.server.init.restore backup_id=value, server_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="opsworkscm",
        operation="restore_server",
        op_kwargs={
            "BackupId": backup_id,
            "ServerName": server_name,
            "InstanceType": instance_type,
            "KeyPair": key_pair,
        },
    )


async def update(
    hub,
    ctx,
    server_name: str,
    disable_automated_backup: bool = None,
    backup_retention_count: int = None,
    preferred_maintenance_window: str = None,
    preferred_backup_window: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Updates settings for a server.   This operation is synchronous.

    Args:
        disable_automated_backup(bool, optional): Setting DisableAutomatedBackup to true disables automated or scheduled backups. Automated backups are enabled by default. . Defaults to None.
        backup_retention_count(int, optional): Sets the number of automated backups that you want to keep. . Defaults to None.
        server_name(str): The name of the server to update. .
        preferred_maintenance_window(str, optional):  DDD:HH:MM (weekly start time) or HH:MM (daily start time).   Time windows always use coordinated universal time (UTC). Valid strings for day of week (DDD) are: Mon, Tue, Wed, Thr, Fri, Sat, or Sun. Defaults to None.
        preferred_backup_window(str, optional):  DDD:HH:MM (weekly start time) or HH:MM (daily start time).   Time windows always use coordinated universal time (UTC). Valid strings for day of week (DDD) are: Mon, Tue, Wed, Thr, Fri, Sat, or Sun. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.opsworkscm.server.init.update(ctx, server_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.opsworkscm.server.init.update server_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="opsworkscm",
        operation="update_server",
        op_kwargs={
            "DisableAutomatedBackup": disable_automated_backup,
            "BackupRetentionCount": backup_retention_count,
            "ServerName": server_name,
            "PreferredMaintenanceWindow": preferred_maintenance_window,
            "PreferredBackupWindow": preferred_backup_window,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def list_all(
    hub,
    ctx,
    grant_arns: List = None,
    filters: List = None,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists grants that are received but not accepted.

    Args:
        grant_arns(List, optional): Amazon Resource Names (ARNs) of the grants. Defaults to None.
        filters(List, optional): Filters to scope the results. The following filters are supported:    ProductSKU     LicenseIssuerName     LicenseArn     GrantStatus     GranterAccountId   . Defaults to None.
        next_token(str, optional): Token for the next set of results. Defaults to None.
        max_results(int, optional): Maximum number of results to return in a single call. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.license_manager.received.grant.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.license_manager.received.grant.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="license-manager",
        operation="list_received_grants",
        op_kwargs={
            "GrantArns": grant_arns,
            "Filters": filters,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(
    hub,
    ctx,
    pipeline_name: str,
    filter_: Dict = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the action executions that have occurred in a pipeline.

    Args:
        pipeline_name(str):  The name of the pipeline for which you want to list action execution history.
        filter_(Dict, optional): Input information used to filter action execution history. Defaults to None.
        max_results(int, optional): The maximum number of results to return in a single call. To retrieve the remaining results, make another call with the returned nextToken value. Action execution history is retained for up to 12 months, based on action execution start times. Default value is 100.   Detailed execution history is available for executions run on or after February 21, 2019. . Defaults to None.
        next_token(str, optional): The token that was returned from the previous ListActionExecutions call, which can be used to return the next set of action executions in the list. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codepipeline.action.execution.list_all(ctx, pipeline_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codepipeline.action.execution.list_all pipeline_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codepipeline",
        operation="list_action_executions",
        op_kwargs={
            "pipelineName": pipeline_name,
            "filter": filter_,
            "maxResults": max_results,
            "nextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def batch(hub, ctx, deployment_id: str, instance_ids: List) -> Dict:
    r"""
    **Autogenerated function**

      This method works, but is deprecated. Use BatchGetDeploymentTargets instead.    Returns an array of one or more instances associated with a deployment. This method works with EC2/On-premises and AWS Lambda compute platforms. The newer BatchGetDeploymentTargets works with all compute platforms. The maximum number of instances that can be returned is 25.

    Args:
        deployment_id(str):  The unique ID of a deployment. .
        instance_ids(List): The unique IDs of instances used in the deployment. The maximum number of instance IDs you can specify is 25.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codedeploy.get.deployment.instance.batch(
                ctx, deployment_id=value, instance_ids=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codedeploy.get.deployment.instance.batch deployment_id=value, instance_ids=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codedeploy",
        operation="batch_get_deployment_instances",
        op_kwargs={"deploymentId": deployment_id, "instanceIds": instance_ids},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(hub, ctx, entries: List) -> Dict:
    r"""
    **Autogenerated function**

    Associates the specified Amazon Web Services Systems Manager document (SSM document) with the specified instances or targets. When you associate a document with one or more instances using instance IDs or tags, Amazon Web Services Systems Manager Agent (SSM Agent) running on the instance processes the document and configures the instance as specified. If you associate a document with an instance that already has an associated document, the system returns the AssociationAlreadyExists exception.

    Args:
        entries(List): One or more associations.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.association.batch.create(ctx, entries=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.association.batch.create entries=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="create_association_batch",
        op_kwargs={"Entries": entries},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__virtualname__ = "by_external_id"


async def delete(hub, ctx, external_id: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Deletes resources linked to an external ID.

    Args:
        external_id(str, optional): The unique ID of an external resource (for example, a CloudFormation stack ID) that is linked to one or more CodeDeploy resources. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codedeploy.resource.by_external_id_.delete(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codedeploy.resource.by_external_id.delete
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codedeploy",
        operation="delete_resources_by_external_id",
        op_kwargs={"externalId": external_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    inline_recipe: bytes = None,
    lambda_function: Dict = None,
    tags: Dict = None,
    client_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a component. Components are software that run on Greengrass core devices. After you develop and test a component on your core device, you can use this operation to upload your component to IoT Greengrass. Then, you can deploy the component to other core devices. You can use this operation to do the following:    Create components from recipes  Create a component from a recipe, which is a file that defines the component's metadata, parameters, dependencies, lifecycle, artifacts, and platform capability. For more information, see IoT Greengrass component recipe reference in the IoT Greengrass V2 Developer Guide. To create a component from a recipe, specify inlineRecipe when you call this operation.    Create components from Lambda functions  Create a component from an Lambda function that runs on IoT Greengrass. This creates a recipe and artifacts from the Lambda function's deployment package. You can use this operation to migrate Lambda functions from IoT Greengrass V1 to IoT Greengrass V2. This function only accepts Lambda functions that use the following runtimes:   Python 2.7 – python2.7    Python 3.7 – python3.7    Python 3.8 – python3.8    Java 8 – java8    Node.js 10 – nodejs10.x    Node.js 12 – nodejs12.x    To create a component from a Lambda function, specify lambdaFunction when you call this operation.

    Args:
        inline_recipe(bytes, optional): The recipe to use to create the component. The recipe defines the component's metadata, parameters, dependencies, lifecycle, artifacts, and platform compatibility. You must specify either inlineRecipe or lambdaFunction. Defaults to None.
        lambda_function(Dict, optional): The parameters to create a component from a Lambda function. You must specify either inlineRecipe or lambdaFunction. Defaults to None.
        tags(Dict, optional): A list of key-value pairs that contain metadata for the resource. For more information, see Tag your resources in the IoT Greengrass V2 Developer Guide. Defaults to None.
        client_token(str, optional): A unique, case-sensitive identifier that you can provide to ensure that the request is idempotent. Idempotency means that the request is successfully processed only once, even if you send the request multiple times. When a request succeeds, and you specify the same client token for subsequent successful requests, the IoT Greengrass V2 service returns the successful response that it caches from the previous request. IoT Greengrass V2 caches successful responses for idempotent requests for up to 8 hours. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.greengrassv2.component.version.init.create(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.greengrassv2.component.version.init.create
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="greengrassv2",
        operation="create_component_version",
        op_kwargs={
            "inlineRecipe": inline_recipe,
            "lambdaFunction": lambda_function,
            "tags": tags,
            "clientToken": client_token,
        },
    )


async def list_all(
    hub, ctx, arn: str, max_results: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves a paginated list of all versions for a component. Greater versions are listed first.

    Args:
        arn(str): The ARN of the component version.
        max_results(int, optional): The maximum number of results to be returned per paginated request. Defaults to None.
        next_token(str, optional): The token to be used for the next set of paginated results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.greengrassv2.component.version.init.list_all(ctx, arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.greengrassv2.component.version.init.list_all arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="greengrassv2",
        operation="list_component_versions",
        op_kwargs={"arn": arn, "maxResults": max_results, "nextToken": next_token},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(
    hub,
    ctx,
    event_integration_name: str,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a paginated list of event integration associations in the account.

    Args:
        event_integration_name(str): The name of the event integration. .
        next_token(str, optional): The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results. Defaults to None.
        max_results(int, optional): The maximum number of results to return per page. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.appintegration.event_integration.association.list_all(
                ctx, event_integration_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.appintegration.event_integration.association.list_all event_integration_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="appintegrations",
        operation="list_event_integration_associations",
        op_kwargs={
            "EventIntegrationName": event_integration_name,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub, ctx, resource_arn: str, max_results: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists tags for an Amazon FSx file systems and backups in the case of Amazon FSx for Windows File Server. When retrieving all tags, you can optionally specify the MaxResults parameter to limit the number of tags in a response. If more tags remain, Amazon FSx returns a NextToken value in the response. In this case, send a later request with the NextToken request parameter set to the value of NextToken from the last response. This action is used in an iterative process to retrieve a list of your tags. ListTagsForResource is called first without a NextTokenvalue. Then the action continues to be called with the NextToken parameter set to the value of the last NextToken value until a response has no NextToken. When using this action, keep the following in mind:   The implementation might return fewer than MaxResults file system descriptions while still including a NextToken value.   The order of tags returned in the response of one ListTagsForResource call and the order of tags returned across the responses of a multi-call iteration is unspecified.

    Args:
        resource_arn(str): The ARN of the Amazon FSx resource that will have its tags listed.
        max_results(int, optional): Maximum number of tags to return in the response (integer). This parameter value must be greater than 0. The number of items that Amazon FSx returns is the minimum of the MaxResults parameter specified in the request and the service's internal maximum number of items per page. Defaults to None.
        next_token(str, optional): Opaque pagination token returned from a previous ListTagsForResource operation (String). If a token present, the action continues the list from where the returning call left off. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.fsx.tags_for_resource.list(ctx, resource_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.fsx.tags_for_resource.list resource_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="fsx",
        operation="list_tags_for_resource",
        op_kwargs={
            "ResourceARN": resource_arn,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""


async def signal(
    hub, ctx, stack_name: str, logical_resource_id: str, unique_id: str, status: str
) -> None:
    r"""
    **Autogenerated function**

    Sends a signal to the specified resource with a success or failure status. You can use the SignalResource API in conjunction with a creation policy or update policy. CloudFormation doesn't proceed with a stack creation or update until resources receive the required number of signals or the timeout period is exceeded. The SignalResource API is useful in cases where you want to send signals from anywhere other than an Amazon EC2 instance.

    Args:
        stack_name(str): The stack name or unique stack ID that includes the resource that you want to signal.
        logical_resource_id(str): The logical ID of the resource that you want to signal. The logical ID is the name of the resource that given in the template.
        unique_id(str): A unique ID of the signal. When you signal Amazon EC2 instances or Auto Scaling groups, specify the instance ID that you are signaling as the unique ID. If you send multiple signals to a single resource (such as signaling a wait condition), each signal requires a different unique ID.
        status(str): The status of the signal, which is either success or failure. A failure signal causes CloudFormation to immediately fail the stack creation or update.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudformation.resource.signal(
                ctx, stack_name=value, logical_resource_id=value, unique_id=value, status=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudformation.resource.signal stack_name=value, logical_resource_id=value, unique_id=value, status=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudformation",
        operation="signal_resource",
        op_kwargs={
            "StackName": stack_name,
            "LogicalResourceId": logical_resource_id,
            "UniqueId": unique_id,
            "Status": status,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__sub_alias__ = ["instance_type_", "instance_type"]


async def list_all(
    hub,
    ctx,
    elasticsearch_version: str,
    domain_name: str = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    List all Elasticsearch instance types that are supported for given ElasticsearchVersion

    Args:
        elasticsearch_version(str): Version of Elasticsearch for which list of supported elasticsearch instance types are needed. .
        domain_name(str, optional): DomainName represents the name of the Domain that we are trying to modify. This should be present only if we are querying for list of available Elasticsearch instance types when modifying existing domain. . Defaults to None.
        max_results(int, optional):  Set this value to limit the number of results returned. Value provided must be greater than 30 else it wont be honored. . Defaults to None.
        next_token(str, optional): NextToken should be sent in case if earlier API call produced result containing NextToken. It is used for pagination. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.es.elasticsearch.instance_type_.init.list_all(
                ctx, elasticsearch_version=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.es.elasticsearch.instance_type_.init.list_all elasticsearch_version=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="es",
        operation="list_elasticsearch_instance_types",
        op_kwargs={
            "ElasticsearchVersion": elasticsearch_version,
            "DomainName": domain_name,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    platform_application_arn: str,
    token: str,
    custom_user_data: str = None,
    attributes: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates an endpoint for a device and mobile app on one of the supported push notification services, such as GCM (Firebase Cloud Messaging) and APNS. CreatePlatformEndpoint requires the PlatformApplicationArn that is returned from CreatePlatformApplication. You can use the returned EndpointArn to send a message to a mobile app or by the Subscribe action for subscription to a topic. The CreatePlatformEndpoint action is idempotent, so if the requester already owns an endpoint with the same device token and attributes, that endpoint's ARN is returned without creating a new endpoint. For more information, see Using Amazon SNS Mobile Push Notifications.  When using CreatePlatformEndpoint with Baidu, two attributes must be provided: ChannelId and UserId. The token field must also contain the ChannelId. For more information, see Creating an Amazon SNS Endpoint for Baidu.

    Args:
        platform_application_arn(str): PlatformApplicationArn returned from CreatePlatformApplication is used to create a an endpoint.
        token(str): Unique identifier created by the notification service for an app on a device. The specific name for Token will vary, depending on which notification service is being used. For example, when using APNS as the notification service, you need the device token. Alternatively, when using GCM (Firebase Cloud Messaging) or ADM, the device token equivalent is called the registration ID.
        custom_user_data(str, optional): Arbitrary user data to associate with the endpoint. Amazon SNS does not use this data. The data must be in UTF-8 format and less than 2KB. Defaults to None.
        attributes(Dict, optional): For a list of attributes, see SetEndpointAttributes. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sns.platform.endpoint.create(
                ctx, platform_application_arn=value, token=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sns.platform.endpoint.create platform_application_arn=value, token=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sns",
        operation="create_platform_endpoint",
        op_kwargs={
            "PlatformApplicationArn": platform_application_arn,
            "Token": token,
            "CustomUserData": custom_user_data,
            "Attributes": attributes,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""


async def associate(hub, ctx, target_distribution_id: str, alias: str) -> None:
    r"""
    **Autogenerated function**

    Associates an alias (also known as a CNAME or an alternate domain name) with a CloudFront distribution. With this operation you can move an alias that’s already in use on a CloudFront distribution to a different distribution in one step. This prevents the downtime that could occur if you first remove the alias from one distribution and then separately add the alias to another distribution. To use this operation to associate an alias with a distribution, you provide the alias and the ID of the target distribution for the alias. For more information, including how to set up the target distribution, prerequisites that you must complete, and other restrictions, see Moving an alternate domain name to a different distribution in the Amazon CloudFront Developer Guide.

    Args:
        target_distribution_id(str): The ID of the distribution that you’re associating the alias with.
        alias(str): The alias (also known as a CNAME) to add to the target distribution.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.alias.associate(
                ctx, target_distribution_id=value, alias=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.alias.associate target_distribution_id=value, alias=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="associate_alias",
        op_kwargs={"TargetDistributionId": target_distribution_id, "Alias": alias},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def update(
    hub, ctx, assessment_id: str, control_set_id: str, status: str, comment: str
) -> Dict:
    r"""
    **Autogenerated function**

     Updates the status of a control set in an Audit Manager assessment.

    Args:
        assessment_id(str):  The identifier for the specified assessment. .
        control_set_id(str):  The identifier for the specified control set. .
        status(str):  The status of the control set that is being updated. .
        comment(str):  The comment related to the status update. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.auditmanager.assessment.control.set_status.update(
                ctx, assessment_id=value, control_set_id=value, status=value, comment=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.auditmanager.assessment.control.set_status.update assessment_id=value, control_set_id=value, status=value, comment=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="auditmanager",
        operation="update_assessment_control_set_status",
        op_kwargs={
            "assessmentId": assessment_id,
            "controlSetId": control_set_id,
            "status": status,
            "comment": comment,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub,
    ctx,
    cluster_identifier: str,
    database: str,
    connected_database: str = None,
    db_user: str = None,
    max_results: int = None,
    next_token: str = None,
    schema_pattern: str = None,
    secret_arn: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the schemas in a database. A token is returned to page through the schema list. Depending on the authorization method, use one of the following combinations of request parameters:    Secrets Manager - specify the Amazon Resource Name (ARN) of the secret, the database name, and the cluster identifier that matches the cluster in the secret.    Temporary credentials - specify the cluster identifier, the database name, and the database user name. Permission to call the redshift:GetClusterCredentials operation is required to use this method.

    Args:
        cluster_identifier(str): The cluster identifier. This parameter is required when authenticating using either Secrets Manager or temporary credentials. .
        connected_database(str, optional): A database name. The connected database is specified when you connect with your authentication credentials. . Defaults to None.
        database(str): The name of the database that contains the schemas to list. If ConnectedDatabase is not specified, this is also the database to connect to with your authentication credentials.
        db_user(str, optional): The database user name. This parameter is required when authenticating using temporary credentials. . Defaults to None.
        max_results(int, optional): The maximum number of schemas to return in the response. If more schemas exist than fit in one response, then NextToken is returned to page through the results. . Defaults to None.
        next_token(str, optional): A value that indicates the starting point for the next set of response records in a subsequent request. If a value is returned in a response, you can retrieve the next set of records by providing this returned NextToken value in the next NextToken parameter and retrying the command. If the NextToken field is empty, all response records have been retrieved for the request. . Defaults to None.
        schema_pattern(str, optional): A pattern to filter results by schema name. Within a schema pattern, "%" means match any substring of 0 or more characters and "_" means match any one character. Only schema name entries matching the search pattern are returned. . Defaults to None.
        secret_arn(str, optional): The name or ARN of the secret that enables access to the database. This parameter is required when authenticating using Secrets Manager. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.redshift_data.schemas.list(
                ctx, cluster_identifier=value, database=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.redshift_data.schemas.list cluster_identifier=value, database=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="redshift-data",
        operation="list_schemas",
        op_kwargs={
            "ClusterIdentifier": cluster_identifier,
            "ConnectedDatabase": connected_database,
            "Database": database,
            "DbUser": db_user,
            "MaxResults": max_results,
            "NextToken": next_token,
            "SchemaPattern": schema_pattern,
            "SecretArn": secret_arn,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""


async def accept(hub, ctx, certificate_id: str, set_as_active: bool = None) -> None:
    r"""
    **Autogenerated function**

    Accepts a pending certificate transfer. The default state of the certificate is INACTIVE. To check for pending certificate transfers, call ListCertificates to enumerate your certificates.

    Args:
        certificate_id(str): The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.).
        set_as_active(bool, optional): Specifies whether the certificate is active. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.certificate.transfer.accept(ctx, certificate_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.certificate.transfer.accept certificate_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="accept_certificate_transfer",
        op_kwargs={"certificateId": certificate_id, "setAsActive": set_as_active},
    )


async def cancel(hub, ctx, certificate_id: str) -> None:
    r"""
    **Autogenerated function**

    Cancels a pending transfer for the specified certificate.  Note Only the transfer source account can use this operation to cancel a transfer. (Transfer destinations can use RejectCertificateTransfer instead.) After transfer, AWS IoT returns the certificate to the source account in the INACTIVE state. After the destination account has accepted the transfer, the transfer cannot be cancelled. After a certificate transfer is cancelled, the status of the certificate changes from PENDING_TRANSFER to INACTIVE.

    Args:
        certificate_id(str): The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.).
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.certificate.transfer.cancel(ctx, certificate_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.certificate.transfer.cancel certificate_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="cancel_certificate_transfer",
        op_kwargs={"certificateId": certificate_id},
    )


async def reject(hub, ctx, certificate_id: str, reject_reason: str = None) -> None:
    r"""
    **Autogenerated function**

    Rejects a pending certificate transfer. After AWS IoT rejects a certificate transfer, the certificate status changes from PENDING_TRANSFER to INACTIVE. To check for pending certificate transfers, call ListCertificates to enumerate your certificates. This operation can only be called by the transfer destination. After it is called, the certificate will be returned to the source's account in the INACTIVE state.

    Args:
        certificate_id(str): The ID of the certificate. (The last part of the certificate ARN contains the certificate ID.).
        reject_reason(str, optional): The reason the certificate transfer was rejected. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.certificate.transfer.reject(ctx, certificate_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.certificate.transfer.reject certificate_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="reject_certificate_transfer",
        op_kwargs={"certificateId": certificate_id, "rejectReason": reject_reason},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def delete(hub, ctx, certificate_id: str, user_name: str = None) -> None:
    r"""
    **Autogenerated function**

    Deletes a signing certificate associated with the specified IAM user. If you do not specify a user name, IAM determines the user name implicitly based on the Amazon Web Services access key ID signing the request. This operation works for access keys under the account. Consequently, you can use this operation to manage account root user credentials even if the account has no associated IAM users.

    Args:
        user_name(str, optional): The name of the user the signing certificate belongs to. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-. Defaults to None.
        certificate_id(str): The ID of the signing certificate to delete. The format of this parameter, as described by its regex pattern, is a string of characters that can be upper- or lower-cased letters or digits.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.signing_certificate.delete(ctx, certificate_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.signing_certificate.delete certificate_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="delete_signing_certificate",
        op_kwargs={"UserName": user_name, "CertificateId": certificate_id},
    )


async def list_all(
    hub, ctx, user_name: str = None, marker: str = None, max_items: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Returns information about the signing certificates associated with the specified IAM user. If none exists, the operation returns an empty list. Although each user is limited to a small number of signing certificates, you can still paginate the results using the MaxItems and Marker parameters. If the UserName field is not specified, the user name is determined implicitly based on the Amazon Web Services access key ID used to sign the request for this operation. This operation works for access keys under the account. Consequently, you can use this operation to manage account root user credentials even if the account has no associated users.

    Args:
        user_name(str, optional): The name of the IAM user whose signing certificates you want to examine. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-. Defaults to None.
        marker(str, optional): Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the Marker element in the response that you received to indicate where the next call should start. Defaults to None.
        max_items(int, optional): Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the IsTruncated response element is true. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the IsTruncated response element returns true, and Marker contains a value to include in the subsequent call that tells the service where to continue from. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.signing_certificate.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.signing_certificate.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="list_signing_certificates",
        op_kwargs={"UserName": user_name, "Marker": marker, "MaxItems": max_items},
    )


async def update(
    hub, ctx, certificate_id: str, status: str, user_name: str = None
) -> None:
    r"""
    **Autogenerated function**

    Changes the status of the specified user signing certificate from active to disabled, or vice versa. This operation can be used to disable an IAM user's signing certificate as part of a certificate rotation work flow. If the UserName field is not specified, the user name is determined implicitly based on the Amazon Web Services access key ID used to sign the request. This operation works for access keys under the account. Consequently, you can use this operation to manage account root user credentials even if the account has no associated users.

    Args:
        user_name(str, optional): The name of the IAM user the signing certificate belongs to. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-. Defaults to None.
        certificate_id(str): The ID of the signing certificate you want to update. This parameter allows (through its regex pattern) a string of characters that can consist of any upper or lowercased letter or digit.
        status(str):  The status you want to assign to the certificate. Active means that the certificate can be used for programmatic calls to Amazon Web Services Inactive means that the certificate cannot be used.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.signing_certificate.update(
                ctx, certificate_id=value, status=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.signing_certificate.update certificate_id=value, status=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="update_signing_certificate",
        op_kwargs={
            "UserName": user_name,
            "CertificateId": certificate_id,
            "Status": status,
        },
    )


async def upload(hub, ctx, certificate_body: str, user_name: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Uploads an X.509 signing certificate and associates it with the specified IAM user. Some Amazon Web Services services require you to use certificates to validate requests that are signed with a corresponding private key. When you upload the certificate, its default status is Active. For information about when you would use an X.509 signing certificate, see Managing server certificates in IAM in the IAM User Guide. If the UserName is not specified, the IAM user name is determined implicitly based on the Amazon Web Services access key ID used to sign the request. This operation works for access keys under the account. Consequently, you can use this operation to manage account root user credentials even if the account has no associated users.  Because the body of an X.509 certificate can be large, you should use POST rather than GET when calling UploadSigningCertificate. For information about setting up signatures and authorization through the API, see Signing Amazon Web Services API requests in the Amazon Web Services General Reference. For general information about using the Query API with IAM, see Making query requests in the IAM User Guide.

    Args:
        user_name(str, optional): The name of the user the signing certificate is for. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-. Defaults to None.
        certificate_body(str): The contents of the signing certificate. The regex pattern used to validate this parameter is a string of characters consisting of the following:   Any printable ASCII character ranging from the space character (\u0020) through the end of the ASCII character range   The printable characters in the Basic Latin and Latin-1 Supplement character set (through \u00FF)   The special characters tab (\u0009), line feed (\u000A), and carriage return (\u000D)  .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.signing_certificate.upload(ctx, certificate_body=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.signing_certificate.upload certificate_body=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="upload_signing_certificate",
        op_kwargs={"UserName": user_name, "CertificateBody": certificate_body},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub,
    ctx,
    instance_arn: str,
    account_id: str,
    provisioning_status: str = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists all the permission sets that are provisioned to a specified Amazon Web Services account.

    Args:
        instance_arn(str): The ARN of the SSO instance under which the operation will be executed. For more information about ARNs, see Amazon Resource Names (ARNs) and Amazon Web Services Service Namespaces in the Amazon Web Services General Reference.
        account_id(str): The identifier of the Amazon Web Services account from which to list the assignments.
        provisioning_status(str, optional): The status object for the permission set provisioning operation. Defaults to None.
        max_results(int, optional): The maximum number of results to display for the assignment. Defaults to None.
        next_token(str, optional): The pagination token for the list API. Initially the value is null. Use the output of previous API calls to make subsequent calls. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sso_admin.permission.sets_provisioned_to_account.list(
                ctx, instance_arn=value, account_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sso_admin.permission.sets_provisioned_to_account.list instance_arn=value, account_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sso-admin",
        operation="list_permission_sets_provisioned_to_account",
        op_kwargs={
            "InstanceArn": instance_arn,
            "AccountId": account_id,
            "ProvisioningStatus": provisioning_status,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(hub, ctx, parameter_group_name: str, description: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Creates a new parameter group. A parameter group is a collection of parameters that you apply to all of the nodes in a DAX cluster.

    Args:
        parameter_group_name(str): The name of the parameter group to apply to all of the clusters in this replication group.
        description(str, optional): A description of the parameter group. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dax.parameter.group.create(ctx, parameter_group_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dax.parameter.group.create parameter_group_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dax",
        operation="create_parameter_group",
        op_kwargs={
            "ParameterGroupName": parameter_group_name,
            "Description": description,
        },
    )


async def delete(hub, ctx, parameter_group_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified parameter group. You cannot delete a parameter group if it is associated with any DAX clusters.

    Args:
        parameter_group_name(str): The name of the parameter group to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dax.parameter.group.delete(ctx, parameter_group_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dax.parameter.group.delete parameter_group_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dax",
        operation="delete_parameter_group",
        op_kwargs={"ParameterGroupName": parameter_group_name},
    )


async def describe_all(
    hub,
    ctx,
    parameter_group_names: List = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of parameter group descriptions. If a parameter group name is specified, the list will contain only the descriptions for that group.

    Args:
        parameter_group_names(List, optional): The names of the parameter groups. Defaults to None.
        max_results(int, optional): The maximum number of results to include in the response. If more results exist than the specified MaxResults value, a token is included in the response so that the remaining results can be retrieved. The value for MaxResults must be between 20 and 100. Defaults to None.
        next_token(str, optional): An optional token returned from a prior request. Use this token for pagination of results from this action. If this parameter is specified, the response includes only results beyond the token, up to the value specified by MaxResults. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dax.parameter.group.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dax.parameter.group.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dax",
        operation="describe_parameter_groups",
        op_kwargs={
            "ParameterGroupNames": parameter_group_names,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )


async def update(
    hub, ctx, parameter_group_name: str, parameter_name_values: List
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the parameters of a parameter group. You can modify up to 20 parameters in a single request by submitting a list parameter name and value pairs.

    Args:
        parameter_group_name(str): The name of the parameter group.
        parameter_name_values(List): An array of name-value pairs for the parameters in the group. Each element in the array represents a single parameter.   record-ttl-millis and query-ttl-millis are the only supported parameter names. For more details, see Configuring TTL Settings. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dax.parameter.group.update(
                ctx, parameter_group_name=value, parameter_name_values=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dax.parameter.group.update parameter_group_name=value, parameter_name_values=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dax",
        operation="update_parameter_group",
        op_kwargs={
            "ParameterGroupName": parameter_group_name,
            "ParameterNameValues": parameter_name_values,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


__func_alias__ = {"import_": "import"}


async def copy(
    hub,
    ctx,
    source_region: str,
    source_snapshot_id: str,
    description: str = None,
    destination_outpost_arn: str = None,
    destination_region: str = None,
    encrypted: bool = None,
    kms_key_id: str = None,
    presigned_url: str = None,
    tag_specifications: List = None,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Copies a point-in-time snapshot of an EBS volume and stores it in Amazon S3. You can copy a snapshot within the same Region, from one Region to another, or from a Region to an Outpost. You can't copy a snapshot from an Outpost to a Region, from one Outpost to another, or within the same Outpost. You can use the snapshot to create EBS volumes or Amazon Machine Images (AMIs). When copying snapshots to a Region, copies of encrypted EBS snapshots remain encrypted. Copies of unencrypted snapshots remain unencrypted, unless you enable encryption for the snapshot copy operation. By default, encrypted snapshot copies use the default Key Management Service (KMS) KMS key; however, you can specify a different KMS key. To copy an encrypted snapshot that has been shared from another account, you must have permissions for the KMS key used to encrypt the snapshot. Snapshots copied to an Outpost are encrypted by default using the default encryption key for the Region, or a different key that you specify in the request using KmsKeyId. Outposts do not support unencrypted snapshots. For more information,  Amazon EBS local snapshots on Outposts in the Amazon Elastic Compute Cloud User Guide. Snapshots created by copying another snapshot have an arbitrary volume ID that should not be used for any purpose. For more information, see Copy an Amazon EBS snapshot in the Amazon Elastic Compute Cloud User Guide.

    Args:
        description(str, optional): A description for the EBS snapshot. Defaults to None.
        destination_outpost_arn(str, optional): The Amazon Resource Name (ARN) of the Outpost to which to copy the snapshot. Only specify this parameter when copying a snapshot from an Amazon Web Services Region to an Outpost. The snapshot must be in the Region for the destination Outpost. You cannot copy a snapshot from an Outpost to a Region, from one Outpost to another, or within the same Outpost. For more information, see  Copy snapshots from an Amazon Web Services Region to an Outpost in the Amazon Elastic Compute Cloud User Guide. Defaults to None.
        destination_region(str, optional): The destination Region to use in the PresignedUrl parameter of a snapshot copy operation. This parameter is only valid for specifying the destination Region in a PresignedUrl parameter, where it is required. The snapshot copy is sent to the regional endpoint that you sent the HTTP request to (for example, ec2.us-east-1.amazonaws.com). With the CLI, this is specified using the --region parameter or the default Region in your Amazon Web Services configuration file. Defaults to None.
        encrypted(bool, optional): To encrypt a copy of an unencrypted snapshot if encryption by default is not enabled, enable encryption using this parameter. Otherwise, omit this parameter. Encrypted snapshots are encrypted, even if you omit this parameter and encryption by default is not enabled. You cannot set this parameter to false. For more information, see Amazon EBS encryption in the Amazon Elastic Compute Cloud User Guide. Defaults to None.
        kms_key_id(str, optional): The identifier of the Key Management Service (KMS) KMS key to use for Amazon EBS encryption. If this parameter is not specified, your KMS key for Amazon EBS is used. If KmsKeyId is specified, the encrypted state must be true. You can specify the KMS key using any of the following:   Key ID. For example, 1234abcd-12ab-34cd-56ef-1234567890ab.   Key alias. For example, alias/ExampleAlias.   Key ARN. For example, arn:aws:kms:us-east-1:012345678910:key/1234abcd-12ab-34cd-56ef-1234567890ab.   Alias ARN. For example, arn:aws:kms:us-east-1:012345678910:alias/ExampleAlias.   Amazon Web Services authenticates the KMS key asynchronously. Therefore, if you specify an ID, alias, or ARN that is not valid, the action can appear to complete, but eventually fails. Defaults to None.
        presigned_url(str, optional): When you copy an encrypted source snapshot using the Amazon EC2 Query API, you must supply a pre-signed URL. This parameter is optional for unencrypted snapshots. For more information, see Query requests. The PresignedUrl should use the snapshot source endpoint, the CopySnapshot action, and include the SourceRegion, SourceSnapshotId, and DestinationRegion parameters. The PresignedUrl must be signed using Amazon Web Services Signature Version 4. Because EBS snapshots are stored in Amazon S3, the signing algorithm for this parameter uses the same logic that is described in Authenticating Requests: Using Query Parameters (Amazon Web Services Signature Version 4) in the Amazon Simple Storage Service API Reference. An invalid or improperly signed PresignedUrl will cause the copy operation to fail asynchronously, and the snapshot will move to an error state. Defaults to None.
        source_region(str): The ID of the Region that contains the snapshot to be copied.
        source_snapshot_id(str): The ID of the EBS snapshot to copy.
        tag_specifications(List, optional): The tags to apply to the new snapshot. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.snapshot.init.copy(ctx, source_region=value, source_snapshot_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.snapshot.init.copy source_region=value, source_snapshot_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="copy_snapshot",
        op_kwargs={
            "Description": description,
            "DestinationOutpostArn": destination_outpost_arn,
            "DestinationRegion": destination_region,
            "Encrypted": encrypted,
            "KmsKeyId": kms_key_id,
            "PresignedUrl": presigned_url,
            "SourceRegion": source_region,
            "SourceSnapshotId": source_snapshot_id,
            "TagSpecifications": tag_specifications,
            "DryRun": dry_run,
        },
    )


async def create(
    hub,
    ctx,
    volume_id: str,
    description: str = None,
    outpost_arn: str = None,
    tag_specifications: List = None,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a snapshot of an EBS volume and stores it in Amazon S3. You can use snapshots for backups, to make copies of EBS volumes, and to save data before shutting down an instance. You can create snapshots of volumes in a Region and volumes on an Outpost. If you create a snapshot of a volume in a Region, the snapshot must be stored in the same Region as the volume. If you create a snapshot of a volume on an Outpost, the snapshot can be stored on the same Outpost as the volume, or in the Region for that Outpost. When a snapshot is created, any Amazon Web Services Marketplace product codes that are associated with the source volume are propagated to the snapshot. You can take a snapshot of an attached volume that is in use. However, snapshots only capture data that has been written to your Amazon EBS volume at the time the snapshot command is issued; this might exclude any data that has been cached by any applications or the operating system. If you can pause any file systems on the volume long enough to take a snapshot, your snapshot should be complete. However, if you cannot pause all file writes to the volume, you should unmount the volume from within the instance, issue the snapshot command, and then remount the volume to ensure a consistent and complete snapshot. You may remount and use your volume while the snapshot status is pending. To create a snapshot for Amazon EBS volumes that serve as root devices, you should stop the instance before taking the snapshot. Snapshots that are taken from encrypted volumes are automatically encrypted. Volumes that are created from encrypted snapshots are also automatically encrypted. Your encrypted volumes and any associated snapshots always remain protected. You can tag your snapshots during creation. For more information, see Tag your Amazon EC2 resources in the Amazon Elastic Compute Cloud User Guide. For more information, see Amazon Elastic Block Store and Amazon EBS encryption in the Amazon Elastic Compute Cloud User Guide.

    Args:
        description(str, optional): A description for the snapshot. Defaults to None.
        outpost_arn(str, optional): The Amazon Resource Name (ARN) of the Outpost on which to create a local snapshot.   To create a snapshot of a volume in a Region, omit this parameter. The snapshot is created in the same Region as the volume.   To create a snapshot of a volume on an Outpost and store the snapshot in the Region, omit this parameter. The snapshot is created in the Region for the Outpost.   To create a snapshot of a volume on an Outpost and store the snapshot on an Outpost, specify the ARN of the destination Outpost. The snapshot must be created on the same Outpost as the volume.   For more information, see Create local snapshots from volumes on an Outpost in the Amazon Elastic Compute Cloud User Guide. Defaults to None.
        volume_id(str): The ID of the Amazon EBS volume.
        tag_specifications(List, optional): The tags to apply to the snapshot during creation. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.snapshot.init.create(ctx, volume_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.snapshot.init.create volume_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="create_snapshot",
        op_kwargs={
            "Description": description,
            "OutpostArn": outpost_arn,
            "VolumeId": volume_id,
            "TagSpecifications": tag_specifications,
            "DryRun": dry_run,
        },
    )


async def create_multiple(
    hub,
    ctx,
    instance_specification: Dict,
    description: str = None,
    outpost_arn: str = None,
    tag_specifications: List = None,
    dry_run: bool = None,
    copy_tags_from_source: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates crash-consistent snapshots of multiple EBS volumes and stores the data in S3. Volumes are chosen by specifying an instance. Any attached volumes will produce one snapshot each that is crash-consistent across the instance. Boot volumes can be excluded by changing the parameters.  You can create multi-volume snapshots of instances in a Region and instances on an Outpost. If you create snapshots from an instance in a Region, the snapshots must be stored in the same Region as the instance. If you create snapshots from an instance on an Outpost, the snapshots can be stored on the same Outpost as the instance, or in the Region for that Outpost.

    Args:
        description(str, optional):  A description propagated to every snapshot specified by the instance. Defaults to None.
        instance_specification(Dict): The instance to specify which volumes should be included in the snapshots.
        outpost_arn(str, optional): The Amazon Resource Name (ARN) of the Outpost on which to create the local snapshots.   To create snapshots from an instance in a Region, omit this parameter. The snapshots are created in the same Region as the instance.   To create snapshots from an instance on an Outpost and store the snapshots in the Region, omit this parameter. The snapshots are created in the Region for the Outpost.   To create snapshots from an instance on an Outpost and store the snapshots on an Outpost, specify the ARN of the destination Outpost. The snapshots must be created on the same Outpost as the instance.   For more information, see  Create multi-volume local snapshots from instances on an Outpost in the Amazon Elastic Compute Cloud User Guide. Defaults to None.
        tag_specifications(List, optional): Tags to apply to every snapshot specified by the instance. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        copy_tags_from_source(str, optional): Copies the tags from the specified volume to corresponding snapshot. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.snapshot.init.create_multiple(ctx, instance_specification=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.snapshot.init.create_multiple instance_specification=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="create_snapshots",
        op_kwargs={
            "Description": description,
            "InstanceSpecification": instance_specification,
            "OutpostArn": outpost_arn,
            "TagSpecifications": tag_specifications,
            "DryRun": dry_run,
            "CopyTagsFromSource": copy_tags_from_source,
        },
    )


async def delete(hub, ctx, snapshot_id: str, dry_run: bool = None) -> None:
    r"""
    **Autogenerated function**

    Deletes the specified snapshot. When you make periodic snapshots of a volume, the snapshots are incremental, and only the blocks on the device that have changed since your last snapshot are saved in the new snapshot. When you delete a snapshot, only the data not needed for any other snapshot is removed. So regardless of which prior snapshots have been deleted, all active snapshots will have access to all the information needed to restore the volume. You cannot delete a snapshot of the root device of an EBS volume used by a registered AMI. You must first de-register the AMI before you can delete the snapshot. For more information, see Delete an Amazon EBS snapshot in the Amazon Elastic Compute Cloud User Guide.

    Args:
        snapshot_id(str): The ID of the EBS snapshot.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.snapshot.init.delete(ctx, snapshot_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.snapshot.init.delete snapshot_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="delete_snapshot",
        op_kwargs={"SnapshotId": snapshot_id, "DryRun": dry_run},
    )


async def describe_all(
    hub,
    ctx,
    filters: List = None,
    max_results: int = None,
    next_token: str = None,
    owner_ids: List = None,
    restorable_by_user_ids: List = None,
    snapshot_ids: List = None,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Describes the specified EBS snapshots available to you or all of the EBS snapshots available to you. The snapshots available to you include public snapshots, private snapshots that you own, and private snapshots owned by other Amazon Web Services accounts for which you have explicit create volume permissions. The create volume permissions fall into the following categories:    public: The owner of the snapshot granted create volume permissions for the snapshot to the all group. All Amazon Web Services accounts have create volume permissions for these snapshots.    explicit: The owner of the snapshot granted create volume permissions to a specific Amazon Web Services account.    implicit: An Amazon Web Services account has implicit create volume permissions for all snapshots it owns.   The list of snapshots returned can be filtered by specifying snapshot IDs, snapshot owners, or Amazon Web Services accounts with create volume permissions. If no options are specified, Amazon EC2 returns all snapshots for which you have create volume permissions. If you specify one or more snapshot IDs, only snapshots that have the specified IDs are returned. If you specify an invalid snapshot ID, an error is returned. If you specify a snapshot ID for which you do not have access, it is not included in the returned results. If you specify one or more snapshot owners using the OwnerIds option, only snapshots from the specified owners and for which you have access are returned. The results can include the Amazon Web Services account IDs of the specified owners, amazon for snapshots owned by Amazon, or self for snapshots that you own. If you specify a list of restorable users, only snapshots with create snapshot permissions for those users are returned. You can specify Amazon Web Services account IDs (if you own the snapshots), self for snapshots for which you own or have explicit permissions, or all for public snapshots. If you are describing a long list of snapshots, we recommend that you paginate the output to make the list more manageable. The MaxResults parameter sets the maximum number of results returned in a single page. If the list of results exceeds your MaxResults value, then that number of results is returned along with a NextToken value that can be passed to a subsequent DescribeSnapshots request to retrieve the remaining results. To get the state of fast snapshot restores for a snapshot, use DescribeFastSnapshotRestores. For more information about EBS snapshots, see Amazon EBS snapshots in the Amazon Elastic Compute Cloud User Guide.

    Args:
        filters(List, optional): The filters.    description - A description of the snapshot.    encrypted - Indicates whether the snapshot is encrypted (true | false)    owner-alias - The owner alias, from an Amazon-maintained list (amazon). This is not the user-configured Amazon Web Services account alias set using the IAM console. We recommend that you use the related parameter instead of this filter.    owner-id - The Amazon Web Services account ID of the owner. We recommend that you use the related parameter instead of this filter.    progress - The progress of the snapshot, as a percentage (for example, 80%).    snapshot-id - The snapshot ID.    start-time - The time stamp when the snapshot was initiated.    status - The status of the snapshot (pending | completed | error).    tag:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key Owner and the value TeamA, specify tag:Owner for the filter name and TeamA for the filter value.    tag-key - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.    volume-id - The ID of the volume the snapshot is for.    volume-size - The size of the volume, in GiB.  . Defaults to None.
        max_results(int, optional): The maximum number of snapshot results returned by DescribeSnapshots in paginated output. When this parameter is used, DescribeSnapshots only returns MaxResults results in a single page along with a NextToken response element. The remaining results of the initial request can be seen by sending another DescribeSnapshots request with the returned NextToken value. This value can be between 5 and 1,000; if MaxResults is given a value larger than 1,000, only 1,000 results are returned. If this parameter is not used, then DescribeSnapshots returns all results. You cannot specify this parameter and the snapshot IDs parameter in the same request. Defaults to None.
        next_token(str, optional): The NextToken value returned from a previous paginated DescribeSnapshots request where MaxResults was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the NextToken value. This value is null when there are no more results to return. Defaults to None.
        owner_ids(List, optional): Scopes the results to snapshots with the specified owners. You can specify a combination of Amazon Web Services account IDs, self, and amazon. Defaults to None.
        restorable_by_user_ids(List, optional): The IDs of the Amazon Web Services accounts that can create volumes from the snapshot. Defaults to None.
        snapshot_ids(List, optional): The snapshot IDs. Default: Describes the snapshots for which you have create volume permissions. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.snapshot.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.snapshot.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="describe_snapshots",
        op_kwargs={
            "Filters": filters,
            "MaxResults": max_results,
            "NextToken": next_token,
            "OwnerIds": owner_ids,
            "RestorableByUserIds": restorable_by_user_ids,
            "SnapshotIds": snapshot_ids,
            "DryRun": dry_run,
        },
    )


async def import_(
    hub,
    ctx,
    client_data: Dict = None,
    client_token: str = None,
    description: str = None,
    disk_container: Dict = None,
    dry_run: bool = None,
    encrypted: bool = None,
    kms_key_id: str = None,
    role_name: str = None,
    tag_specifications: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Imports a disk into an EBS snapshot. For more information, see Importing a disk as a snapshot using VM Import/Export in the VM Import/Export User Guide.

    Args:
        client_data(Dict, optional): The client-specific data. Defaults to None.
        client_token(str, optional): Token to enable idempotency for VM import requests. Defaults to None.
        description(str, optional): The description string for the import snapshot task. Defaults to None.
        disk_container(Dict, optional): Information about the disk container. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        encrypted(bool, optional): Specifies whether the destination snapshot of the imported image should be encrypted. The default CMK for EBS is used unless you specify a non-default AWS Key Management Service (AWS KMS) CMK using KmsKeyId. For more information, see Amazon EBS Encryption in the Amazon Elastic Compute Cloud User Guide. Defaults to None.
        kms_key_id(str, optional): An identifier for the symmetric AWS Key Management Service (AWS KMS) customer master key (CMK) to use when creating the encrypted snapshot. This parameter is only required if you want to use a non-default CMK; if this parameter is not specified, the default CMK for EBS is used. If a KmsKeyId is specified, the Encrypted flag must also be set.  The CMK identifier may be provided in any of the following formats:    Key ID   Key alias. The alias ARN contains the arn:aws:kms namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the alias namespace, and then the CMK alias. For example, arn:aws:kms:us-east-1:012345678910:alias/ExampleAlias.   ARN using key ID. The ID ARN contains the arn:aws:kms namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the key namespace, and then the CMK ID. For example, arn:aws:kms:us-east-1:012345678910:key/abcd1234-a123-456a-a12b-a123b4cd56ef.   ARN using key alias. The alias ARN contains the arn:aws:kms namespace, followed by the Region of the CMK, the AWS account ID of the CMK owner, the alias namespace, and then the CMK alias. For example, arn:aws:kms:us-east-1:012345678910:alias/ExampleAlias.    AWS parses KmsKeyId asynchronously, meaning that the action you call may appear to complete even though you provided an invalid identifier. This action will eventually report failure.  The specified CMK must exist in the Region that the snapshot is being copied to. Amazon EBS does not support asymmetric CMKs. Defaults to None.
        role_name(str, optional): The name of the role to use when not using the default role, 'vmimport'. Defaults to None.
        tag_specifications(List, optional): The tags to apply to the import snapshot task during creation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.snapshot.init.import(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.snapshot.init.import
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="import_snapshot",
        op_kwargs={
            "ClientData": client_data,
            "ClientToken": client_token,
            "Description": description,
            "DiskContainer": disk_container,
            "DryRun": dry_run,
            "Encrypted": encrypted,
            "KmsKeyId": kms_key_id,
            "RoleName": role_name,
            "TagSpecifications": tag_specifications,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def accept(
    hub, ctx, dry_run: bool = None, vpc_peering_connection_id: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Accept a VPC peering connection request. To accept a request, the VPC peering connection must be in the pending-acceptance state, and you must be the owner of the peer VPC. Use DescribeVpcPeeringConnections to view your outstanding VPC peering connection requests. For an inter-Region VPC peering connection request, you must accept the VPC peering connection in the Region of the accepter VPC.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        vpc_peering_connection_id(str, optional): The ID of the VPC peering connection. You must specify this parameter in the request. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.vpc.peering_connection.init.accept(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.vpc.peering_connection.init.accept
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="accept_vpc_peering_connection",
        op_kwargs={
            "DryRun": dry_run,
            "VpcPeeringConnectionId": vpc_peering_connection_id,
        },
    )


async def create(
    hub,
    ctx,
    dry_run: bool = None,
    peer_owner_id: str = None,
    peer_vpc_id: str = None,
    vpc_id: str = None,
    peer_region: str = None,
    tag_specifications: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Requests a VPC peering connection between two VPCs: a requester VPC that you own and an accepter VPC with which to create the connection. The accepter VPC can belong to another Amazon Web Services account and can be in a different Region to the requester VPC. The requester VPC and accepter VPC cannot have overlapping CIDR blocks.  Limitations and rules apply to a VPC peering connection. For more information, see the limitations section in the VPC Peering Guide.  The owner of the accepter VPC must accept the peering request to activate the peering connection. The VPC peering connection request expires after 7 days, after which it cannot be accepted or rejected. If you create a VPC peering connection request between VPCs with overlapping CIDR blocks, the VPC peering connection has a status of failed.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        peer_owner_id(str, optional): The Amazon Web Services account ID of the owner of the accepter VPC. Default: Your Amazon Web Services account ID. Defaults to None.
        peer_vpc_id(str, optional): The ID of the VPC with which you are creating the VPC peering connection. You must specify this parameter in the request. Defaults to None.
        vpc_id(str, optional): The ID of the requester VPC. You must specify this parameter in the request. Defaults to None.
        peer_region(str, optional): The Region code for the accepter VPC, if the accepter VPC is located in a Region other than the Region in which you make the request. Default: The Region in which you make the request. Defaults to None.
        tag_specifications(List, optional): The tags to assign to the peering connection. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.vpc.peering_connection.init.create(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.vpc.peering_connection.init.create
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="create_vpc_peering_connection",
        op_kwargs={
            "DryRun": dry_run,
            "PeerOwnerId": peer_owner_id,
            "PeerVpcId": peer_vpc_id,
            "VpcId": vpc_id,
            "PeerRegion": peer_region,
            "TagSpecifications": tag_specifications,
        },
    )


async def delete(
    hub, ctx, vpc_peering_connection_id: str, dry_run: bool = None
) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a VPC peering connection. Either the owner of the requester VPC or the owner of the accepter VPC can delete the VPC peering connection if it's in the active state. The owner of the requester VPC can delete a VPC peering connection in the pending-acceptance state. You cannot delete a VPC peering connection that's in the failed state.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        vpc_peering_connection_id(str): The ID of the VPC peering connection.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.vpc.peering_connection.init.delete(
                ctx, vpc_peering_connection_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.vpc.peering_connection.init.delete vpc_peering_connection_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="delete_vpc_peering_connection",
        op_kwargs={
            "DryRun": dry_run,
            "VpcPeeringConnectionId": vpc_peering_connection_id,
        },
    )


async def describe_all(
    hub,
    ctx,
    filters: List = None,
    dry_run: bool = None,
    vpc_peering_connection_ids: List = None,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Describes one or more of your VPC peering connections.

    Args:
        filters(List, optional): One or more filters.    accepter-vpc-info.cidr-block - The IPv4 CIDR block of the accepter VPC.    accepter-vpc-info.owner-id - The ID of the Amazon Web Services account that owns the accepter VPC.    accepter-vpc-info.vpc-id - The ID of the accepter VPC.    expiration-time - The expiration date and time for the VPC peering connection.    requester-vpc-info.cidr-block - The IPv4 CIDR block of the requester's VPC.    requester-vpc-info.owner-id - The ID of the Amazon Web Services account that owns the requester VPC.    requester-vpc-info.vpc-id - The ID of the requester VPC.    status-code - The status of the VPC peering connection (pending-acceptance | failed | expired | provisioning | active | deleting | deleted | rejected).    status-message - A message that provides more information about the status of the VPC peering connection, if applicable.    tag:<key> - The key/value combination of a tag assigned to the resource. Use the tag key in the filter name and the tag value as the filter value. For example, to find all resources that have a tag with the key Owner and the value TeamA, specify tag:Owner for the filter name and TeamA for the filter value.    tag-key - The key of a tag assigned to the resource. Use this filter to find all resources assigned a tag with a specific key, regardless of the tag value.    vpc-peering-connection-id - The ID of the VPC peering connection.  . Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        vpc_peering_connection_ids(List, optional): One or more VPC peering connection IDs. Default: Describes all your VPC peering connections. Defaults to None.
        next_token(str, optional): The token for the next page of results. Defaults to None.
        max_results(int, optional): The maximum number of results to return with a single call. To retrieve the remaining results, make another call with the returned nextToken value. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.vpc.peering_connection.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.vpc.peering_connection.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="describe_vpc_peering_connections",
        op_kwargs={
            "Filters": filters,
            "DryRun": dry_run,
            "VpcPeeringConnectionIds": vpc_peering_connection_ids,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )


async def reject(
    hub, ctx, vpc_peering_connection_id: str, dry_run: bool = None
) -> Dict:
    r"""
    **Autogenerated function**

    Rejects a VPC peering connection request. The VPC peering connection must be in the pending-acceptance state. Use the DescribeVpcPeeringConnections request to view your outstanding VPC peering connection requests. To delete an active VPC peering connection, or to delete a VPC peering connection request that you initiated, use DeleteVpcPeeringConnection.

    Args:
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        vpc_peering_connection_id(str): The ID of the VPC peering connection.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.vpc.peering_connection.init.reject(
                ctx, vpc_peering_connection_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.vpc.peering_connection.init.reject vpc_peering_connection_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="reject_vpc_peering_connection",
        op_kwargs={
            "DryRun": dry_run,
            "VpcPeeringConnectionId": vpc_peering_connection_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def modify(
    hub,
    ctx,
    cluster_identifier: str,
    add_iam_roles: List = None,
    remove_iam_roles: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the list of Identity and Access Management (IAM) roles that can be used by the cluster to access other Amazon Web Services services. A cluster can have up to 10 IAM roles associated at any time.

    Args:
        cluster_identifier(str): The unique identifier of the cluster for which you want to associate or disassociate IAM roles.
        add_iam_roles(List, optional): Zero or more IAM roles to associate with the cluster. The roles must be in their Amazon Resource Name (ARN) format. You can associate up to 10 IAM roles with a single cluster in a single request. Defaults to None.
        remove_iam_roles(List, optional): Zero or more IAM roles in ARN format to disassociate from the cluster. You can disassociate up to 10 IAM roles from a single cluster in a single request. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.redshift.cluster.iam_role.modify(ctx, cluster_identifier=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.redshift.cluster.iam_role.modify cluster_identifier=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="redshift",
        operation="modify_cluster_iam_roles",
        op_kwargs={
            "ClusterIdentifier": cluster_identifier,
            "AddIamRoles": add_iam_roles,
            "RemoveIamRoles": remove_iam_roles,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub, ctx, code_signing_config_arn: str, marker: str = None, max_items: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    List the functions that use the specified code signing configuration. You can use this method prior to deleting a code signing configuration, to verify that no functions are using it.

    Args:
        code_signing_config_arn(str): The The Amazon Resource Name (ARN) of the code signing configuration.
        marker(str, optional): Specify the pagination token that's returned by a previous request to retrieve the next page of results. Defaults to None.
        max_items(int, optional): Maximum number of items to return. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lambda_.function.by_code_signing_config.list(
                ctx, code_signing_config_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lambda_.function.by_code_signing_config.list code_signing_config_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lambda",
        operation="list_functions_by_code_signing_config",
        op_kwargs={
            "CodeSigningConfigArn": code_signing_config_arn,
            "Marker": marker,
            "MaxItems": max_items,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get_all(
    hub,
    ctx,
    detector_id: str,
    finding_statistic_types: List,
    finding_criteria: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists Amazon GuardDuty findings statistics for the specified detector ID.

    Args:
        detector_id(str): The ID of the detector that specifies the GuardDuty service whose findings' statistics you want to retrieve.
        finding_statistic_types(List): The types of finding statistics to retrieve.
        finding_criteria(Dict, optional): Represents the criteria that is used for querying findings. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.guardduty.finding.statistic.get_all(
                ctx, detector_id=value, finding_statistic_types=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.guardduty.finding.statistic.get_all detector_id=value, finding_statistic_types=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="guardduty",
        operation="get_findings_statistics",
        op_kwargs={
            "DetectorId": detector_id,
            "FindingStatisticTypes": finding_statistic_types,
            "FindingCriteria": finding_criteria,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def update(
    hub,
    ctx,
    firewall_policy_change_protection: bool,
    update_token: str = None,
    firewall_arn: str = None,
    firewall_name: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Args:
        update_token(str, optional): An optional token that you can use for optimistic locking. Network Firewall returns a token to your requests that access the firewall. The token marks the state of the firewall resource at the time of the request.  To make an unconditional change to the firewall, omit the token in your update request. Without the token, Network Firewall performs your updates regardless of whether the firewall has changed since you last retrieved it. To make a conditional change to the firewall, provide the token in your update request. Network Firewall uses the token to ensure that the firewall hasn't changed since you last retrieved it. If it has changed, the operation fails with an InvalidTokenException. If this happens, retrieve the firewall again to get a current copy of it with a new token. Reapply your changes as needed, then try the operation again using the new token. . Defaults to None.
        firewall_arn(str, optional): The Amazon Resource Name (ARN) of the firewall. You must specify the ARN or the name, and you can specify both. . Defaults to None.
        firewall_name(str, optional): The descriptive name of the firewall. You can't change the name of a firewall after you create it. You must specify the ARN or the name, and you can specify both. . Defaults to None.
        firewall_policy_change_protection(bool): A setting indicating whether the firewall is protected against a change to the firewall policy association. Use this setting to protect against accidentally modifying the firewall policy for a firewall that is in use. When you create a firewall, the operation initializes this setting to TRUE.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.network_firewall.firewall.policy.change_protection.update(
                ctx, firewall_policy_change_protection=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.network_firewall.firewall.policy.change_protection.update firewall_policy_change_protection=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="network-firewall",
        operation="update_firewall_policy_change_protection",
        op_kwargs={
            "UpdateToken": update_token,
            "FirewallArn": firewall_arn,
            "FirewallName": firewall_name,
            "FirewallPolicyChangeProtection": firewall_policy_change_protection,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub,
    ctx,
    bucket: str,
    delimiter: str = None,
    encoding_type: str = None,
    max_keys: int = None,
    prefix: str = None,
    continuation_token: str = None,
    fetch_owner: bool = None,
    start_after: str = None,
    request_payer: str = None,
    expected_bucket_owner: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns some or all (up to 1,000) of the objects in a bucket with each request. You can use the request parameters as selection criteria to return a subset of the objects in a bucket. A 200 OK response can contain valid or invalid XML. Make sure to design your application to parse the contents of the response and handle it appropriately. Objects are returned sorted in an ascending order of the respective key names in the list. For more information about listing objects, see Listing object keys programmatically  To use this operation, you must have READ access to the bucket. To use this action in an AWS Identity and Access Management (IAM) policy, you must have permissions to perform the s3:ListBucket action. The bucket owner has this permission by default and can grant this permission to others. For more information about permissions, see Permissions Related to Bucket Subresource Operations and Managing Access Permissions to Your Amazon S3 Resources.  This section describes the latest revision of this action. We recommend that you use this revised API for application development. For backward compatibility, Amazon S3 continues to support the prior version of this API, ListObjects.  To get a list of your buckets, see ListBuckets. The following operations are related to ListObjectsV2:    GetObject     PutObject     CreateBucket

    Args:
        bucket(str): Bucket name to list.  When using this action with an access point, you must direct requests to the access point hostname. The access point hostname takes the form AccessPointName-AccountId.s3-accesspoint.Region.amazonaws.com. When using this action with an access point through the AWS SDKs, you provide the access point ARN in place of the bucket name. For more information about access point ARNs, see Using access points in the Amazon S3 User Guide. When using this action with Amazon S3 on Outposts, you must direct requests to the S3 on Outposts hostname. The S3 on Outposts hostname takes the form AccessPointName-AccountId.outpostID.s3-outposts.Region.amazonaws.com. When using this action using S3 on Outposts through the AWS SDKs, you provide the Outposts bucket ARN in place of the bucket name. For more information about S3 on Outposts ARNs, see Using S3 on Outposts in the Amazon S3 User Guide.
        delimiter(str, optional): A delimiter is a character you use to group keys. Defaults to None.
        encoding_type(str, optional): Encoding type used by Amazon S3 to encode object keys in the response. Defaults to None.
        max_keys(int, optional): Sets the maximum number of keys returned in the response. By default the action returns up to 1,000 key names. The response might contain fewer keys but will never contain more. Defaults to None.
        prefix(str, optional): Limits the response to keys that begin with the specified prefix. Defaults to None.
        continuation_token(str, optional): ContinuationToken indicates Amazon S3 that the list is being continued on this bucket with a token. ContinuationToken is obfuscated and is not a real key. Defaults to None.
        fetch_owner(bool, optional): The owner field is not present in listV2 by default, if you want to return owner field with each key in the result then set the fetch owner field to true. Defaults to None.
        start_after(str, optional): StartAfter is where you want Amazon S3 to start listing from. Amazon S3 starts listing after this specified key. StartAfter can be any key in the bucket. Defaults to None.
        request_payer(str, optional): Confirms that the requester knows that she or he will be charged for the list objects request in V2 style. Bucket owners need not specify this parameter in their requests. Defaults to None.
        expected_bucket_owner(str, optional): The account ID of the expected bucket owner. If the bucket is owned by a different account, the request will fail with an HTTP 403 (Access Denied) error. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.s3.object_.v2.list(ctx, bucket=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.s3.object_.v2.list bucket=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="s3",
        operation="list_objects_v2",
        op_kwargs={
            "Bucket": bucket,
            "Delimiter": delimiter,
            "EncodingType": encoding_type,
            "MaxKeys": max_keys,
            "Prefix": prefix,
            "ContinuationToken": continuation_token,
            "FetchOwner": fetch_owner,
            "StartAfter": start_after,
            "RequestPayer": request_payer,
            "ExpectedBucketOwner": expected_bucket_owner,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    domain: str,
    repository: str,
    domain_owner: str = None,
    description: str = None,
    upstreams: List = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Creates a repository.

    Args:
        domain(str):  The name of the domain that contains the created repository. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        repository(str):  The name of the repository to create. .
        description(str, optional):  A description of the created repository. . Defaults to None.
        upstreams(List, optional):  A list of upstream repositories to associate with the repository. The order of the upstream repositories in the list determines their priority order when AWS CodeArtifact looks for a requested package version. For more information, see Working with upstream repositories. . Defaults to None.
        tags(List, optional): One or more tag key-value pairs for the repository. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.repository.init.create(
                ctx, domain=value, repository=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.repository.init.create domain=value, repository=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="create_repository",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "repository": repository,
            "description": description,
            "upstreams": upstreams,
            "tags": tags,
        },
    )


async def delete(
    hub, ctx, domain: str, repository: str, domain_owner: str = None
) -> Dict:
    r"""
    **Autogenerated function**

     Deletes a repository.

    Args:
        domain(str):  The name of the domain that contains the repository to delete. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        repository(str):  The name of the repository to delete. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.repository.init.delete(
                ctx, domain=value, repository=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.repository.init.delete domain=value, repository=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="delete_repository",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "repository": repository,
        },
    )


async def describe(
    hub, ctx, domain: str, repository: str, domain_owner: str = None
) -> Dict:
    r"""
    **Autogenerated function**

     Returns a RepositoryDescription object that contains detailed information about the requested repository.

    Args:
        domain(str):  The name of the domain that contains the repository to describe. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        repository(str):  A string that specifies the name of the requested repository. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.repository.init.describe(
                ctx, domain=value, repository=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.repository.init.describe domain=value, repository=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="describe_repository",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "repository": repository,
        },
    )


async def list_all(
    hub,
    ctx,
    repository_prefix: str = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Returns a list of RepositorySummary objects. Each RepositorySummary contains information about a repository in the specified AWS account and that matches the input parameters.

    Args:
        repository_prefix(str, optional):  A prefix used to filter returned repositories. Only repositories with names that start with repositoryPrefix are returned. Defaults to None.
        max_results(int, optional):  The maximum number of results to return per page. . Defaults to None.
        next_token(str, optional):  The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.repository.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.repository.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="list_repositories",
        op_kwargs={
            "repositoryPrefix": repository_prefix,
            "maxResults": max_results,
            "nextToken": next_token,
        },
    )


async def update(
    hub,
    ctx,
    domain: str,
    repository: str,
    domain_owner: str = None,
    description: str = None,
    upstreams: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Update the properties of a repository.

    Args:
        domain(str):  The name of the domain associated with the repository to update. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        repository(str):  The name of the repository to update. .
        description(str, optional):  An updated repository description. . Defaults to None.
        upstreams(List, optional):  A list of upstream repositories to associate with the repository. The order of the upstream repositories in the list determines their priority order when AWS CodeArtifact looks for a requested package version. For more information, see Working with upstream repositories. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.repository.init.update(
                ctx, domain=value, repository=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.repository.init.update domain=value, repository=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="update_repository",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "repository": repository,
            "description": description,
            "upstreams": upstreams,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub, ctx, name: str, sources: List, robot_software_suite: Dict, tags: Dict = None
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a robot application.

    Args:
        name(str): The name of the robot application.
        sources(List): The sources of the robot application.
        robot_software_suite(Dict): The robot software suite (ROS distribuition) used by the robot application.
        tags(Dict, optional): A map that contains tag keys and tag values that are attached to the robot application. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.robot.application.init.create(
                ctx, name=value, sources=value, robot_software_suite=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.robot.application.init.create name=value, sources=value, robot_software_suite=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="create_robot_application",
        op_kwargs={
            "name": name,
            "sources": sources,
            "robotSoftwareSuite": robot_software_suite,
            "tags": tags,
        },
    )


async def delete(hub, ctx, application: str, application_version: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a robot application.

    Args:
        application(str): The Amazon Resource Name (ARN) of the the robot application.
        application_version(str, optional): The version of the robot application to delete. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.robot.application.init.delete(ctx, application=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.robot.application.init.delete application=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="delete_robot_application",
        op_kwargs={
            "application": application,
            "applicationVersion": application_version,
        },
    )


async def describe(hub, ctx, application: str, application_version: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Describes a robot application.

    Args:
        application(str): The Amazon Resource Name (ARN) of the robot application.
        application_version(str, optional): The version of the robot application to describe. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.robot.application.init.describe(
                ctx, application=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.robot.application.init.describe application=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="describe_robot_application",
        op_kwargs={
            "application": application,
            "applicationVersion": application_version,
        },
    )


async def list_all(
    hub,
    ctx,
    version_qualifier: str = None,
    next_token: str = None,
    max_results: int = None,
    filters: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of robot application. You can optionally provide filters to retrieve specific robot applications.

    Args:
        version_qualifier(str, optional): The version qualifier of the robot application. Defaults to None.
        next_token(str, optional): If the previous paginated request did not return all of the remaining results, the response object's nextToken parameter value is set to a token. To retrieve the next set of results, call ListRobotApplications again and assign that token to the request object's nextToken parameter. If there are no remaining results, the previous response object's NextToken parameter is set to null. . Defaults to None.
        max_results(int, optional): When this parameter is used, ListRobotApplications only returns maxResults results in a single page along with a nextToken response element. The remaining results of the initial request can be seen by sending another ListRobotApplications request with the returned nextToken value. This value can be between 1 and 100. If this parameter is not used, then ListRobotApplications returns up to 100 results and a nextToken value if applicable. . Defaults to None.
        filters(List, optional): Optional filters to limit results. The filter name name is supported. When filtering, you must use the complete value of the filtered item. You can use up to three filters. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.robot.application.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.robot.application.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="list_robot_applications",
        op_kwargs={
            "versionQualifier": version_qualifier,
            "nextToken": next_token,
            "maxResults": max_results,
            "filters": filters,
        },
    )


async def update(
    hub,
    ctx,
    application: str,
    sources: List,
    robot_software_suite: Dict,
    current_revision_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates a robot application.

    Args:
        application(str): The application information for the robot application.
        sources(List): The sources of the robot application.
        robot_software_suite(Dict): The robot software suite (ROS distribution) used by the robot application.
        current_revision_id(str, optional): The revision id for the robot application. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.robot.application.init.update(
                ctx, application=value, sources=value, robot_software_suite=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.robot.application.init.update application=value, sources=value, robot_software_suite=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="update_robot_application",
        op_kwargs={
            "application": application,
            "sources": sources,
            "robotSoftwareSuite": robot_software_suite,
            "currentRevisionId": current_revision_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub,
    ctx,
    workteam_arn: str,
    max_results: int = None,
    next_token: str = None,
    creation_time_after: str = None,
    creation_time_before: str = None,
    job_reference_code_contains: str = None,
    sort_by: str = None,
    sort_order: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Gets a list of labeling jobs assigned to a specified work team.

    Args:
        workteam_arn(str): The Amazon Resource Name (ARN) of the work team for which you want to see labeling jobs for.
        max_results(int, optional): The maximum number of labeling jobs to return in each page of the response. Defaults to None.
        next_token(str, optional): If the result of the previous ListLabelingJobsForWorkteam request was truncated, the response includes a NextToken. To retrieve the next set of labeling jobs, use the token in the next request. Defaults to None.
        creation_time_after(str, optional): A filter that returns only labeling jobs created after the specified time (timestamp). Defaults to None.
        creation_time_before(str, optional): A filter that returns only labeling jobs created before the specified time (timestamp). Defaults to None.
        job_reference_code_contains(str, optional): A filter the limits jobs to only the ones whose job reference code contains the specified string. Defaults to None.
        sort_by(str, optional): The field to sort results by. The default is CreationTime. Defaults to None.
        sort_order(str, optional): The sort order for results. The default is Ascending. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.labeling_jobs_for_workteam.list(
                ctx, workteam_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.labeling_jobs_for_workteam.list workteam_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="list_labeling_jobs_for_workteam",
        op_kwargs={
            "WorkteamArn": workteam_arn,
            "MaxResults": max_results,
            "NextToken": next_token,
            "CreationTimeAfter": creation_time_after,
            "CreationTimeBefore": creation_time_before,
            "JobReferenceCodeContains": job_reference_code_contains,
            "SortBy": sort_by,
            "SortOrder": sort_order,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub,
    ctx,
    name: str,
    metadata: str,
    document_version: str = None,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Information about approval reviews for a version of an SSM document.

    Args:
        name(str): The name of the document.
        document_version(str, optional): The version of the document. Defaults to None.
        metadata(str): The type of data for which details are being requested. Currently, the only supported value is DocumentReviews.
        next_token(str, optional): The token for the next set of items to return. (You received this token from a previous call.). Defaults to None.
        max_results(int, optional): The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.document.metadata.history.list(
                ctx, name=value, metadata=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.document.metadata.history.list name=value, metadata=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="list_document_metadata_history",
        op_kwargs={
            "Name": name,
            "DocumentVersion": document_version,
            "Metadata": metadata,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"set_": "set"}


async def set_(hub, ctx, gateway_arn: str, password: str) -> Dict:
    r"""
    **Autogenerated function**

    Sets the password for the guest user smbguest. The smbguest user is the user when the authentication method for the file share is set to GuestAccess. This operation only supported for S3 File Gateways

    Args:
        gateway_arn(str): The Amazon Resource Name (ARN) of the S3 File Gateway the SMB file share is associated with.
        password(str): The password that you want to set for your SMB server.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.storagegateway.smb.guest_password.set(
                ctx, gateway_arn=value, password=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.storagegateway.smb.guest_password.set gateway_arn=value, password=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="storagegateway",
        operation="set_smb_guest_password",
        op_kwargs={"GatewayARN": gateway_arn, "Password": password},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    account_id: str,
    budget_name: str,
    notification_type: str,
    action_type: str,
    action_threshold: Dict,
    definition: Dict,
    execution_role_arn: str,
    approval_model: str,
    subscribers: List,
) -> Dict:
    r"""
    **Autogenerated function**

     Creates a budget action.

    Args:
        account_id(str): The account ID of the user. It should be a 12-digit number.
        budget_name(str):  A string that represents the budget name. The ":" and "\" characters aren't allowed.
        notification_type(str):  The type of a notification. It must be ACTUAL or FORECASTED.
        action_type(str):  The type of action. This defines the type of tasks that can be carried out by this action. This field also determines the format for definition. .
        action_threshold(Dict):  The trigger threshold of the action. .
        definition(Dict):  Specifies all of the type-specific parameters. .
        execution_role_arn(str):  The role passed for action execution and reversion. Roles and actions must be in the same account. .
        approval_model(str):  This specifies if the action needs manual or automatic approval. .
        subscribers(List):  A list of subscribers.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.budget.budget.action.init.create(
                ctx,
                account_id=value,
                budget_name=value,
                notification_type=value,
                action_type=value,
                action_threshold=value,
                definition=value,
                execution_role_arn=value,
                approval_model=value,
                subscribers=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.budget.budget.action.init.create account_id=value, budget_name=value, notification_type=value, action_type=value, action_threshold=value, definition=value, execution_role_arn=value, approval_model=value, subscribers=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="budgets",
        operation="create_budget_action",
        op_kwargs={
            "AccountId": account_id,
            "BudgetName": budget_name,
            "NotificationType": notification_type,
            "ActionType": action_type,
            "ActionThreshold": action_threshold,
            "Definition": definition,
            "ExecutionRoleArn": execution_role_arn,
            "ApprovalModel": approval_model,
            "Subscribers": subscribers,
        },
    )


async def delete(hub, ctx, account_id: str, budget_name: str, action_id: str) -> Dict:
    r"""
    **Autogenerated function**

     Deletes a budget action.

    Args:
        account_id(str): The account ID of the user. It should be a 12-digit number.
        budget_name(str):  A string that represents the budget name. The ":" and "\" characters aren't allowed.
        action_id(str):  A system-generated universally unique identifier (UUID) for the action. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.budget.budget.action.init.delete(
                ctx, account_id=value, budget_name=value, action_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.budget.budget.action.init.delete account_id=value, budget_name=value, action_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="budgets",
        operation="delete_budget_action",
        op_kwargs={
            "AccountId": account_id,
            "BudgetName": budget_name,
            "ActionId": action_id,
        },
    )


async def describe(hub, ctx, account_id: str, budget_name: str, action_id: str) -> Dict:
    r"""
    **Autogenerated function**

     Describes a budget action detail.

    Args:
        account_id(str): The account ID of the user. It should be a 12-digit number.
        budget_name(str):  A string that represents the budget name. The ":" and "\" characters aren't allowed.
        action_id(str):  A system-generated universally unique identifier (UUID) for the action. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.budget.budget.action.init.describe(
                ctx, account_id=value, budget_name=value, action_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.budget.budget.action.init.describe account_id=value, budget_name=value, action_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="budgets",
        operation="describe_budget_action",
        op_kwargs={
            "AccountId": account_id,
            "BudgetName": budget_name,
            "ActionId": action_id,
        },
    )


async def execute(
    hub, ctx, account_id: str, budget_name: str, action_id: str, execution_type: str
) -> Dict:
    r"""
    **Autogenerated function**

     Executes a budget action.

    Args:
        account_id(str): The account ID of the user. It should be a 12-digit number.
        budget_name(str):  A string that represents the budget name. The ":" and "\" characters aren't allowed.
        action_id(str):  A system-generated universally unique identifier (UUID) for the action. .
        execution_type(str):  The type of execution. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.budget.budget.action.init.execute(
                ctx, account_id=value, budget_name=value, action_id=value, execution_type=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.budget.budget.action.init.execute account_id=value, budget_name=value, action_id=value, execution_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="budgets",
        operation="execute_budget_action",
        op_kwargs={
            "AccountId": account_id,
            "BudgetName": budget_name,
            "ActionId": action_id,
            "ExecutionType": execution_type,
        },
    )


async def update(
    hub,
    ctx,
    account_id: str,
    budget_name: str,
    action_id: str,
    notification_type: str = None,
    action_threshold: Dict = None,
    definition: Dict = None,
    execution_role_arn: str = None,
    approval_model: str = None,
    subscribers: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Updates a budget action.

    Args:
        account_id(str): The account ID of the user. It should be a 12-digit number.
        budget_name(str):  A string that represents the budget name. The ":" and "\" characters aren't allowed.
        action_id(str):  A system-generated universally unique identifier (UUID) for the action. .
        notification_type(str, optional):  The type of a notification. It must be ACTUAL or FORECASTED. Defaults to None.
        action_threshold(Dict, optional):  The trigger threshold of the action. . Defaults to None.
        definition(Dict, optional):  Specifies all of the type-specific parameters. . Defaults to None.
        execution_role_arn(str, optional):  The role passed for action execution and reversion. Roles and actions must be in the same account. . Defaults to None.
        approval_model(str, optional):  This specifies if the action needs manual or automatic approval. . Defaults to None.
        subscribers(List, optional):  A list of subscribers. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.budget.budget.action.init.update(
                ctx, account_id=value, budget_name=value, action_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.budget.budget.action.init.update account_id=value, budget_name=value, action_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="budgets",
        operation="update_budget_action",
        op_kwargs={
            "AccountId": account_id,
            "BudgetName": budget_name,
            "ActionId": action_id,
            "NotificationType": notification_type,
            "ActionThreshold": action_threshold,
            "Definition": definition,
            "ExecutionRoleArn": execution_role_arn,
            "ApprovalModel": approval_model,
            "Subscribers": subscribers,
        },
    )
